/*!
 * @interisk-software/chartjs-plugin-selectdata v0.2.1
 * 
 * (c) 2021 @interisk-software/chartjs-plugin-selectdata contributors
 * Released under the MIT license
 */
(function (global, factory) {
typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('chart.js')) :
typeof define === 'function' && define.amd ? define(['exports', 'chart.js'], factory) :
(global = typeof globalThis !== 'undefined' ? globalThis : global || self, factory(global.ChartSelectData = {}, global.Chart));
}(this, (function (exports, Chart) { 'use strict';

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var Chart__default = /*#__PURE__*/_interopDefaultLegacy(Chart);

const EXPANDO_COLOR = '$oldColor';
const EXPANDO_INDEX = '$selectedIndex';
const EXPANDO_INDEX_DATASET = '$selectedIndexDataSet';
const ALPHA = 0.1;

const {helpers} = Chart__default['default'];

const ifNeedClearSelection = function(dataset, selectedIndex, selectedIndexDataSet, resetKeys) {
  if (dataset[EXPANDO_INDEX] === selectedIndex && dataset[EXPANDO_INDEX_DATASET] === selectedIndexDataSet) {
    dataset[EXPANDO_INDEX] = null;
    dataset[EXPANDO_INDEX_DATASET] = null;
    resetKeys.forEach(key => {
      dataset[key] = dataset[EXPANDO_COLOR][key];
    });
    return true;
  }

  dataset[EXPANDO_INDEX] = selectedIndex;
  dataset[EXPANDO_INDEX_DATASET] = selectedIndexDataSet;
  return false;
};

const checkExpandoKeys = function(dataset, checkKeys) {
  checkKeys.forEach(key => {
    if (dataset[EXPANDO_COLOR] && dataset[EXPANDO_COLOR][key] !== undefined) {
      dataset[key] = dataset[EXPANDO_COLOR][key];
    }
  });
};

const setColorsAlphaNotSelected = function(dataset, selectedIndex, key) {
  if (Array.isArray(dataset[key]) && dataset[key].length) {
    dataset[EXPANDO_COLOR][key] = dataset[key];
    dataset[key] = dataset[key]
      .map((color, colorIndex) => (selectedIndex !== colorIndex ? helpers.color(color).alpha(ALPHA).rgbString() : color));
  }
  if (typeof dataset[key] === 'string') {
    dataset[EXPANDO_COLOR][key] = dataset[key];
    dataset[key] = dataset.data
      .map((e, colorIndex) => (selectedIndex !== colorIndex ? helpers.color(dataset[key]).alpha(ALPHA).rgbString() : dataset[key]));
  }
};

const setColorsAlphaAll = function(dataset, key) {
  if (typeof dataset[key] === 'string') {
    dataset[EXPANDO_COLOR][key] = dataset[key];
    dataset[key] = helpers.color(dataset[key]).alpha(ALPHA).rgbString();
  }
  if (Array.isArray(dataset[key]) && dataset[key].length) {
    dataset[EXPANDO_COLOR][key] = dataset[key];
    dataset[key] = dataset[key].map((color) => helpers.color(color).alpha(ALPHA).rgbString());
  }
};

const selectIndexDataSet = function(chart, selectedIndex, selectedIndexDataSet) {
  chart.selectedIndex = selectedIndex;
  chart.selectIndexDataSet = selectedIndexDataSet;
  let clearSelection = false;
  const indexIsEqual = (index, anotherIndex, trust, fake) => index === anotherIndex ? trust() : fake();
  chart.config.data.datasets.forEach((dataset, indexDataSet) => {
    if (!dataset[EXPANDO_COLOR]) {
      dataset[EXPANDO_COLOR] = {};
    }
    dataset._integrity = true;
    switch (dataset.type || chart.config.type) {
    case 'line':
    case 'radar':
    case 'scatter':
      dataset.pointBackgroundColor = dataset.pointBackgroundColor === undefined ? dataset.borderColor || 'rgb(0,0,0)' : dataset.pointBackgroundColor;
      if (ifNeedClearSelection(dataset, selectedIndex, selectedIndexDataSet, ['pointBackgroundColor', 'borderColor'])) {
        clearSelection = true;
        return;
      }
      checkExpandoKeys(dataset, ['pointBackgroundColor', 'borderColor']);

      indexIsEqual(
        indexDataSet, selectedIndexDataSet,
        () => setColorsAlphaNotSelected(dataset, selectedIndex, 'pointBackgroundColor'),
        ()=> setColorsAlphaAll(dataset, 'pointBackgroundColor')
      );

      setColorsAlphaAll(dataset, 'borderColor');
      break;
    default:
      if (ifNeedClearSelection(dataset, selectedIndex, selectedIndexDataSet, ['backgroundColor'])) {
        clearSelection = true;
        return;
      }
      checkExpandoKeys(dataset, ['backgroundColor']);

      indexIsEqual(
        indexDataSet, selectedIndexDataSet,
        () => setColorsAlphaNotSelected(dataset, selectedIndex, 'backgroundColor'),
        ()=> setColorsAlphaAll(dataset, 'backgroundColor')
      );
      break;
    }
  });

  chart.update();
  return clearSelection;
};

const emitEventSelection = function(selectedIndex, selectedIndexDataSet, clearSelection, options, chart) {
  const params = {
    datasetIndex: selectedIndexDataSet,
    index: selectedIndex,
  };
  if (clearSelection && options.onSelectClear) {
    options.onSelectClear(params, chart);
  }
  if (!clearSelection && options.onSelect) {
    options.onSelect(params, chart);
  }
};

const utils = {
  ifNeedClearSelection,
  checkExpandoKeys,
  setColorsAlphaNotSelected,
  setColorsAlphaAll,
  selectIndexDataSet,
  emitEventSelection
};

const globals = Chart__default['default'].defaults.global || Chart__default['default'].defaults;

globals.plugins.selectdata = {
  onSelect: undefined,
  onSelectClear: undefined
};
const SelectionDataPlugin = {
  id: 'selectdata',
  beforeUpdate(chart) {
    if (chart.data.datasets.find(el => !el._integrity) && chart.selectedIndex !== undefined && chart.selectIndexDataSet !== undefined) {
      utils.selectIndexDataSet(chart, chart.selectedIndex, chart.selectIndexDataSet);
    }
  },
  afterInit(chart) {
    chart.clearSelection = function() {
      chart.config.data.datasets.forEach((dataset) => {
        dataset[EXPANDO_INDEX] = null;
        dataset[EXPANDO_INDEX_DATASET] = null;
        Object.keys(dataset).forEach(key => {
          if (dataset[EXPANDO_COLOR] && dataset[EXPANDO_COLOR][key] !== undefined) {
            dataset[key] = dataset[EXPANDO_COLOR][key];
          }
        });
      });
      chart.update();
    };
    chart.selectDataIndex = function(index, indexDataSet = 0) {
      utils.selectIndexDataSet(chart, index, indexDataSet);
      chart.update();
    };
  },
  afterEvent(chart, chartEvent, options) {
    if (!options) {
      return;
    }
    const eventType = chartEvent.type || chartEvent.event.type;
    if (eventType === 'click') {

      let selectedElements = [];
      if (chart.getElementAtEvent) {
        selectedElements = chart.getElementAtEvent(chartEvent);
      } else {
        selectedElements = chart.getElementsAtEventForMode(chartEvent.event.native, 'nearest', {intersect: true}, false);
      }

      if (!selectedElements || (Array.isArray(selectedElements) && !selectedElements.length)) {
        return;
      }
      const selectedIndex = selectedElements[0]._index === undefined ? selectedElements[0].index : selectedElements[0]._index;
      const selectIndexDataSet = selectedElements[0]._datasetIndex === undefined ? selectedElements[0].datasetIndex : selectedElements[0]._datasetIndex;
      let clearSelection = utils.selectIndexDataSet(chart, selectedIndex, selectIndexDataSet);
      utils.emitEventSelection(selectedIndex, selectIndexDataSet, clearSelection, options, chart);
    }
  },
};

window.SelectionDataPlugin = SelectionDataPlugin;

exports.default = SelectionDataPlugin;

Object.defineProperty(exports, '__esModule', { value: true });

})));
